from functools import wraps
from flask import request, redirect, url_for, g, render_template
from models.user_session import UserSession
from models.user import User
from models.base import Session

# Enhanced authentication middleware
def auth_required(roles=None):
    def decorator(f):
        @wraps(f)
        def decorated_function(*args, **kwargs):
            # First check if any users exist
            session = Session()
            try:
                user_exists = session.query(User).first() is not None
                if not user_exists:
                    return redirect(url_for('auth.register_page'))
            finally:
                session.close()

            # Then check authentication
            auth_token = request.cookies.get('session_token')
            if not auth_token:
                return redirect(url_for('auth.login_page'))
            
            user_id = UserSession.validate_session(auth_token)
            if not user_id:
                return redirect(url_for('auth.login_page'))
            
            user = User.get_by_id(user_id)
            if not user:
                return redirect(url_for('auth.login_page'))
            
            if roles and user.role not in roles:
                return render_template('403.html'), 403
            
            # Add user to request context
            g.user = user
            return f(*args, **kwargs)
        return decorated_function
    return decorator
